// -------------------------------------------------------------------------------- 
//!  @file  stage01.c                                                               
//!  背景ステージ 01 処理のソースファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "bg.h"
#include "enemy/boss0.h"
#include "stage01.h"


// -------------------------------------------------------------------------------- 
//!  背景データ更新タスク VBlank 処理                                               
//!                                                                                 
//!  @param [in,out] p_tcb  タスク TCB                                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC stage01_updateBgData_vblank( TCB *p_tcb )
{
	static const uint8_t	palette[]={
								PALPOS_ARTIFICIALSUN0,
								PALPOS_ARTIFICIALSUN1,
								PALPOS_ARTIFICIALSUN2,
								PALPOS_ARTIFICIALSUN3,
								PALPOS_ARTIFICIALSUN4,
								PALPOS_ARTIFICIALSUN3,
								PALPOS_ARTIFICIALSUN2,
								PALPOS_ARTIFICIALSUN1
							} ;

	// キャラクターデータ
	memcpy(&g.vram.pat[CHRPOS_ARTIFICIALSUN][0], &g.vram.pat[CHRPOS_ARTIFICIALSUN0+(((g.count1/8)&3)*CHRDATA_ARTIFICIALSUN0_COUNT)][0], CHRDATA_ARTIFICIALSUN0_COUNT*32) ;

	// パレットデータ
	memcpy(&g.vram.pal[PALPOS_ARTIFICIALSUN][0], &g.vram.pal[palette[(g.count1/8)&7]][0], 16*2) ;
}


// -------------------------------------------------------------------------------- 
//!  背景データ更新タスク処理                                                       
//!                                                                                 
//!  @param [in,out] p_tcb  タスク TCB                                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC stage01_updateBgData( TCB *p_tcb )
{
	if( !(g.count1&7) ) {
		addVblankProc(stage01_updateBgData_vblank, p_tcb) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  パレット更新 VBlank 処理                                                       
//!                                                                                 
//!  @param [in,out] p_tcb  タスク TCB                                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC stage01_updatePalette_vblank( TCB *p_tcb )
{
	uint32_t	color[4] ;


	color[0] = 5 + (getSin( (g.count1 +  0) & 127) ) * 10 / FP_ONE ;
	color[1] =     (getSin( (g.count1 + 16) & 127) ) * 15 / FP_ONE ;
	color[2] =     (getSin( (g.count2 + 32) & 127) ) * 15 / FP_ONE ;
	color[3] = 5 + (getSin( (g.count1 + 48) & 127) ) * 10 / FP_ONE ;

	// 星パレットデータ
	g.vram.pal[PALPOS_FIX][8]  =  color[0] ;
	g.vram.pal[PALPOS_FIX][9]  = (color[0] << 8) | (color[0] << 4) | (color[0] << 0) ;
	g.vram.pal[PALPOS_FIX][10] =  color[1] << 8 ;
	g.vram.pal[PALPOS_FIX][11] =  color[2] << 4 ;
	g.vram.pal[PALPOS_FIX][12] = (color[1] << 4) | (color[1] << 0) ;
	g.vram.pal[PALPOS_FIX][13] = (color[3] << 8) | (color[3] << 4) ;
}


// -------------------------------------------------------------------------------- 
//!  パレット更新タスク                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  タスク TCB                                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC stage01_updatePalette( TCB *p_tcb )
{
	addVblankProc(stage01_updatePalette_vblank, p_tcb) ;
}


// -------------------------------------------------------------------------------- 
//!  背景ステージ 01 イベント処理                                                   
//!                                                                                 
//!  @param [in,out] p_parent  親タスク TCB                                         
//!  @param [in]     param     パラメータ                                           
//!  @return                   なし                                                 
// -------------------------------------------------------------------------------- 
void RAMFUNC stage01_event(BGTCB *p_parent, uint16_t param)
{
	switch( (uint8_t )param ) {
		case 0:
		{
			// タスク生成
			task_create((TCB *)p_parent, stage01_updateBgData,  TASK_PRIO_BG) ;
			task_create((TCB *)p_parent, stage01_updatePalette, TASK_PRIO_BG) ;
			break ;
		}

		case 2:
		{
			boss0_create(p_parent, param) ;
			break ;
		}
	}
}
