// -------------------------------------------------------------------------------- 
//!  @file  boss0.c                                                                 
//!  ボス 0 タスク処理のヘッダファイル                                              
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "bullet00.h"
#include "bullet01.h"
#include "boss0.h"

	//! TCB 状態
	enum {
		TCB_STATUS_SHIELD0			=	1 << 0,						//!< 遮蔽板０あり
		TCB_STATUS_SHIELD1			=	1 << 1,						//!< 遮蔽板１あり
		TCB_STATUS_SHIELD2			=	1 << 2,						//!< 遮蔽板２あり
		TCB_STATUS_SHIELD3			=	1 << 3,						//!< 遮蔽板３あり
		TCB_STATUS_SHIELD4			=	1 << 4,						//!< 遮蔽板４あり
		TCB_STATUS_SHIELD			=	TCB_STATUS_SHIELD0 | TCB_STATUS_SHIELD1 | TCB_STATUS_SHIELD2 | TCB_STATUS_SHIELD3 | TCB_STATUS_SHIELD4,
																	//!< 遮蔽板あり
		TCB_STATUS_INDESTRUCTIBLE	=	1 << 5						//!< 破壊不可能状態
	} ;


static void RAMFUNC boss0( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_040( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_080( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_0c0( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_100( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_200( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_240( ENEMYTCB *p_tcb ) ;
static void RAMFUNC boss0_280( ENEMYTCB *p_tcb ) ;

static void RAMFUNC core_040( ENEMYTCB *p_tcb ) ;
static void RAMFUNC core_080( ENEMYTCB *p_tcb ) ;
static void RAMFUNC core_0c0( ENEMYTCB *p_tcb ) ;
static void RAMFUNC core_100( ENEMYTCB *p_tcb ) ;

static void RAMFUNC boss0_destroy( ENEMYTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  本体当たり処理                                                                 
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC body_hit(HIT *p_self, HIT *p_rival)
{
	p_rival->proc(p_rival, p_self) ;
}


// -------------------------------------------------------------------------------- 
//!  本体タスク                                                                     
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC body( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  本体タスクを生成                                                               
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @param [in] pattern   パターン番号                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static void RAMFUNC createBody(ENEMYTCB *p_parent, uint16_t pattern)
{
	ENEMYTCB	*p_tcb ;


	p_tcb               = task_create((TCB *)p_parent, body, TASK_PRIO_ENEMY0+1) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 8 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = p_parent->obj.x ;
	p_tcb->obj.y        = p_parent->obj.y ;
	p_tcb->obj.hit.proc = body_hit ;

	obj_setPattern(&p_tcb->obj, pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  コア当たり処理                                                                 
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC core_hit(HIT *p_self, HIT *p_rival)
{
	ENEMYTCB	*p_tcb=MEMBERSTRUCT(ENEMYTCB, obj.hit, p_self) ;


	p_rival->proc(p_rival, p_self) ;

	// 破壊不可能か遮蔽板が残っているなら無視
	if( p_tcb->p_parent->status & (TCB_STATUS_INDESTRUCTIBLE|TCB_STATUS_SHIELD) ) {
		return ;
	}

	// 判定済み
	if( p_self->flag & HIT_FLAG_JUDGE ) {
		return ;
	}
	p_self->flag |= HIT_FLAG_JUDGE ;

	// ダメージ
	p_tcb->shield -= p_rival->strength ;
	if( p_tcb->shield > 0 ) {
		sound_requestSE(3, SE_31) ;
		return ;
	}

	sound_requestSE(3, SE_22) ;

	g.score += 50 ;

	enemy_explosion(p_tcb, ACT_EXPLOSION00, 0) ;

	boss0_destroy( p_tcb->p_parent ) ;
}


// -------------------------------------------------------------------------------- 
//!  コアタスク                                                                     
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC core( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	if( !--p_tcb->timer0 ) {
		p_tcb->taskProc = core_040 ;
		obj_setAction(&p_tcb->obj, ACT_BOSS0_OPENCORE) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, PAT_BOSS0_CORE0) ;
}


static void RAMFUNC core_040( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	obj_updateAction( &p_tcb->obj ) ;

	if( p_tcb->obj.status & OBJ_STATUS_ACTEND ) {
		sound_requestSE(0, SE_52) ;

		p_tcb->p_parent->status &= ~TCB_STATUS_INDESTRUCTIBLE ;

		p_tcb->taskProc = core_080 ;
		p_tcb->timer0   = 2000 ;
		return ;
	}
}


static void RAMFUNC core_080( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	if( !--p_tcb->timer0 ) {
		p_tcb->taskProc = core_0c0 ;
		obj_setAction(&p_tcb->obj, ACT_BOSS0_CLOSECORE) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, PAT_BOSS0_CORE4) ;
}


static void RAMFUNC core_0c0( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	if( p_tcb->obj.status & OBJ_STATUS_ACTEND ) {
		p_tcb->p_parent->status |= TCB_STATUS_INDESTRUCTIBLE ;

		p_tcb->taskProc = core_100 ;
		obj_setPattern(&p_tcb->obj, PAT_BOSS0_CORE0) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


static void RAMFUNC core_100( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	obj_setPattern(&p_tcb->obj, PAT_BOSS0_CORE0) ;
}


// -------------------------------------------------------------------------------- 
//!  コアタスクを生成                                                               
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static void RAMFUNC createCore( ENEMYTCB *p_parent )
{
	ENEMYTCB	*p_tcb ;

	// タスク生成
	p_tcb               = task_create((TCB *)p_parent, core, TASK_PRIO_ENEMY0+1) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 8 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = p_parent->obj.x ;
	p_tcb->obj.y        = p_parent->obj.y ;
	p_tcb->obj.hit.proc = core_hit ;
	p_tcb->shield       = 100 ;
	p_tcb->timer0       = 350 ;

	obj_setPattern(&p_tcb->obj, PAT_BOSS0_CORE0) ;
}


// -------------------------------------------------------------------------------- 
//!  遮蔽板当たり処理                                                               
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC shield_hit(HIT *p_self, HIT *p_rival)
{
	ENEMYTCB	*p_tcb=MEMBERSTRUCT(ENEMYTCB, obj.hit, p_self) ;


	p_rival->proc(p_rival, p_self) ;

	// 破壊不可能
	if( p_tcb->p_parent->status & TCB_STATUS_INDESTRUCTIBLE ) {
		return ;
	}

	// 判定済み
	if( p_self->flag & HIT_FLAG_JUDGE ) {
		return ;
	}
	p_self->flag |= HIT_FLAG_JUDGE ;

	// ダメージ
	p_tcb->shield -= p_rival->strength ;
	if( p_tcb->shield > 0 ) {
		sound_requestSE(3, SE_31) ;

		p_tcb->p_parent->status &= ~(p_tcb->status & TCB_STATUS_SHIELD) ;
		return ;
	}

	sound_requestSE(3, SE_22) ;

	g.score++ ;

	enemy_explosion(p_tcb, ACT_EXPLOSION01, false) ;

	// 弾
	bullet00_create(p_tcb->obj.x, p_tcb->obj.y, 128-32, toFp( 1.5 ), ACT_BULLET00) ;
	bullet00_create(p_tcb->obj.x, p_tcb->obj.y, 128-16, toFp( 1.5 ), ACT_BULLET00) ;
	bullet00_create(p_tcb->obj.x, p_tcb->obj.y, 128+ 0, toFp( 1.5 ), ACT_BULLET00) ;
	bullet00_create(p_tcb->obj.x, p_tcb->obj.y, 128+16, toFp( 1.5 ), ACT_BULLET00) ;
	bullet00_create(p_tcb->obj.x, p_tcb->obj.y, 128+32, toFp( 1.5 ), ACT_BULLET00) ;
}


// -------------------------------------------------------------------------------- 
//!  遮蔽板タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC shield( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x = p_tcb->p_parent->obj.x + p_tcb->xofs ;
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  遮蔽板タスクを生成                                                             
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @param [in] xofs      ｘ座標オフセット                                         
//!  @param [in] pattern   パターン番号                                             
//!  @param [in] status    状態                                                     
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static void RAMFUNC createShield(ENEMYTCB *p_parent, int16_t xofs, uint16_t pattern, uint8_t status)
{
	ENEMYTCB	*p_tcb ;

	// タスク生成
	p_tcb               = task_create((TCB *)p_parent, shield, TASK_PRIO_ENEMY0+1) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 8 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = p_parent->obj.x + toFp( xofs ) ;
	p_tcb->obj.y        = p_parent->obj.y ;
	p_tcb->obj.hit.proc = shield_hit ;
	p_tcb->status      |= status ;
	p_tcb->xofs         = toFp( xofs ) ;
	p_tcb->shield       = 60 ;

	obj_setPattern(&p_tcb->obj, pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  ボス 0 タスク                                                                  
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC boss0( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x -= toFp( 1 ) ;
	if( getInt( p_tcb->obj.x ) > 240 ) {
		return ;
	}

	p_tcb->taskProc = boss0_040 ;
	p_tcb->count0   = 3 ;
	p_tcb->timer0   = 30 ;
}


static void RAMFUNC boss0_040( ENEMYTCB *p_tcb )
{
	if( --p_tcb->timer0 ) {
		return ;
	}

	// 前方移動
	if( (uint32_t )(getInt( g.player.y - p_tcb->obj.y )+10) < (10*2) ) {
		if( !--p_tcb->count0 ) {
			p_tcb->taskProc = boss0_200 ;
			p_tcb->count0   = 3 ;
			p_tcb->obj.y    = g.player.y ;
			return ;
		}
	}

	// 移動方向取得
	if( p_tcb->obj.y >= g.player.y ) {	// 上へ
		p_tcb->angle = -64 ;
	} else {							// 下へ
		p_tcb->angle =  64 ;
	}

	p_tcb->taskProc = boss0_080 ;
	p_tcb->timer0   = 24 ;
}


static void RAMFUNC boss0_080( ENEMYTCB *p_tcb )
{
	if( !--p_tcb->timer0 ) {
		bullet01_create(p_tcb->obj.x-toFp(  4 ), p_tcb->obj.y-toFp( 24 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
		bullet01_create(p_tcb->obj.x-toFp( 32 ), p_tcb->obj.y-toFp( 10 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
		bullet01_create(p_tcb->obj.x-toFp( 32 ), p_tcb->obj.y+toFp( 10 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
		bullet01_create(p_tcb->obj.x-toFp(  4 ), p_tcb->obj.y+toFp( 24 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;

		sound_requestSE(3, SE_08) ;

		p_tcb->timer0   = 30 ;
		p_tcb->taskProc = boss0_040 ;
		return ;
	}

	if( (int8_t )p_tcb->angle < 0 ) {	// 上へ
		p_tcb->obj.y -= toFp( 2 ) ;
	} else {							// 下へ
		p_tcb->obj.y += toFp( 2 ) ;
	}
}


static void RAMFUNC boss0_0c0( ENEMYTCB *p_tcb )
{
	if( --p_tcb->timer0 ) {
		return ;
	}

	sound_requestSE(3, SE_23) ;
	sound_fadeOutBGM() ;

	task_destroyPrio(TASK_PRIO_ENEMY0+1, TASK_PRIO_ENEMY0+1) ;

	p_tcb->taskProc = boss0_100 ;
	p_tcb->timer0   = 120 ;
}


static void RAMFUNC boss0_100( ENEMYTCB *p_tcb )
{
	if( --p_tcb->timer0 ) {
		return ;
	}

	task_setSystemMode( SYSTEM_MODE_GAMEOVER ) ;
}


static void RAMFUNC boss0_200( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x -= toFp( 2 ) ;
	if( getInt( p_tcb->obj.x ) > 180 ) {
		return ;
	}

	bullet01_create(p_tcb->obj.x-toFp(  4 ), p_tcb->obj.y-toFp( 24 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
	bullet01_create(p_tcb->obj.x-toFp( 32 ), p_tcb->obj.y-toFp( 10 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
	bullet01_create(p_tcb->obj.x-toFp( 32 ), p_tcb->obj.y+toFp( 10 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;
	bullet01_create(p_tcb->obj.x-toFp(  4 ), p_tcb->obj.y+toFp( 24 ), 128, toFp( 3 ), PAT_BOSS0_LASER) ;

	sound_requestSE(3, SE_08) ;

	p_tcb->taskProc = boss0_240 ;
	p_tcb->timer0   = 20 ;
}


static void RAMFUNC boss0_240( ENEMYTCB *p_tcb )
{
	if( !--p_tcb->timer0 ) {
		p_tcb->taskProc = boss0_280 ;
	}
}


static void RAMFUNC boss0_280( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x += toFp( 2 ) ;
	if( getInt( p_tcb->obj.x ) < 240 ) {
		return ;
	}

	p_tcb->taskProc = boss0_040 ;
	p_tcb->timer0   = 5 ;
}


// -------------------------------------------------------------------------------- 
//!  ボス 0 タスクを生成                                                            
//!                                                                                 
//!  @param [in] p_parent 親タスク TCB                                              
//!  @param [in] param    パラメータ                                                
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
void RAMFUNC boss0_create(BGTCB *p_parent, uint16_t param )
{
	ENEMYTCB	*p_tcb ;


	bg_stopScroll() ;
	sound_playBGM( BGM_10 ) ;

	// タスク生成
	p_tcb          = task_create(NULL, boss0, TASK_PRIO_ENEMY0) ;
	p_tcb->status |= (TCB_STATUS_INDESTRUCTIBLE | TCB_STATUS_SHIELD) ;
	p_tcb->obj.x   = toFp( SCREEN_WIDTH+128 ) ;
	p_tcb->obj.y   = toFp( SCREEN_HEIGHT/2 ) ;

	// 遮蔽板生成
	createShield(p_tcb, -40, PAT_BOSS0_SHIELD0, TCB_STATUS_SHIELD0) ;
	createShield(p_tcb, -36, PAT_BOSS0_SHIELD0, TCB_STATUS_SHIELD1) ;
	createShield(p_tcb, -32, PAT_BOSS0_SHIELD0, TCB_STATUS_SHIELD2) ;
	createShield(p_tcb, -28, PAT_BOSS0_SHIELD0, TCB_STATUS_SHIELD3) ;
	createShield(p_tcb, -24, PAT_BOSS0_SHIELD1, TCB_STATUS_SHIELD4) ;

	// コア生成
	createCore( p_tcb ) ;

	// 本体生成
	createBody(p_tcb, PAT_BOSS0_BODY0) ;
	createBody(p_tcb, PAT_BOSS0_BODY1) ;
	createBody(p_tcb, PAT_BOSS0_BODY2) ;
	createBody(p_tcb, PAT_BOSS0_BODY3) ;
	createBody(p_tcb, PAT_BOSS0_BODY4) ;
}


// -------------------------------------------------------------------------------- 
//!  ボス 0 破壊                                                                    
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC boss0_destroy( ENEMYTCB *p_tcb )
{
	p_tcb->status  |= TCB_STATUS_INDESTRUCTIBLE ;
	p_tcb->taskProc = boss0_0c0 ;
	p_tcb->timer0   = 60 ;
}
