// -------------------------------------------------------------------------------- 
//!  @file  bullet01.c                                                              
//!  弾 01 タスク処理のソースファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "bullet01.h"


// -------------------------------------------------------------------------------- 
//!  弾 01 当たり処理                                                               
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet01_hit(HIT *p_self, HIT *p_rival)
{
	p_rival->proc(p_rival, p_self) ;
}


// -------------------------------------------------------------------------------- 
//!  弾 01 タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet01( ENEMYTCB *p_tcb )
{
	// ｘ方向
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) - toFp( g.bg.xofs ) ;
	if( (getInt( p_tcb->obj.x )<-8) || (getInt( p_tcb->obj.x )>=(SCREEN_WIDTH+8)) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  弾 01 タスクを生成                                                             
//!                                                                                 
//!  @param [in] x        表示ｘ座標                                                
//!  @param [in] y        表示ｙ座標                                                
//!  @param [in] angle    角度                                                      
//!  @param [in] speed    速度                                                      
//!  @param [in] pattern  パターン番号                                              
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
void RAMFUNC bullet01_create(int24f8_t x, int24f8_t y, uint8_t angle, int8f8_t speed, uint16_t pattern)
{
	ENEMYTCB	*p_tcb ;


	if( !g.enemy.remain ) {
		return ;
	}
	g.enemy.remain-- ;

	// タスク生成
	p_tcb               = task_create(NULL, bullet01, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 0 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = x ;
	p_tcb->obj.y        = y ;
	p_tcb->obj.hit.proc = bullet01_hit ;
	p_tcb->angle        = angle ;
	p_tcb->speed        = speed ;

	obj_setPattern(&p_tcb->obj, pattern) ;
}
