// -------------------------------------------------------------------------------- 
//!  @file  enemy.c                                                                 
//!  敵タスク管理のソースファイル                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "enemy00.h"
#include "enemy01.h"
#include "enemy02.h"
#include "powercapsule.h"


typedef void (*CREATEENEMYPROC)(int24f8_t y, uint16_t param) ;		//!< 敵タスク作成処理


static void RAMFUNC enemy_explosion_040( ENEMYTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  敵タスクを生成                                                                 
//!                                                                                 
//!  @param [in] y      ｙ座標                                                      
//!  @param [in] param  パラメータ                                                  
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy_create(int24f8_t y, uint16_t param)
{
	const CREATEENEMYPROC	createEnemy[]={
								enemy00_create,
								enemy01_create,
								enemy02_create
							} ;


	DBGASSERT( (uint8_t )param < (sizeof( createEnemy )/sizeof( createEnemy[0] )) ) ;

	createEnemy[(uint8_t )param](y, param) ;
}


// -------------------------------------------------------------------------------- 
//!  敵タスクを破棄                                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  敵タスク TCB                                            
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy_destroy( ENEMYTCB *p_tcb )
{
	g.enemy.remain++ ;

	task_destroy( (TCB *)p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  敵タスクの爆発表示                                                             
//!                                                                                 
//!  @param [in,out] p_tcb         敵タスク                                         
//!  @param [in]     action        アクション番号                                   
//!  @param [in]     powerCapsule  アクション終了後のパワーカプセル表示フラグ       
//!                                - true :表示する                                 
//!                                - false:表示しない                               
//!  @return                       なし                                             
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy_explosion(ENEMYTCB *p_tcb, uint8_t action, bool powerCapsule)
{
	p_tcb->taskProc = enemy_explosion_040 ;
	p_tcb->param    = powerCapsule ;

	obj_setAction(&p_tcb->obj, action) ;
}


static void RAMFUNC enemy_explosion_040( ENEMYTCB *p_tcb )
{
	if( p_tcb->obj.status & OBJ_STATUS_ACTEND ) {
		if( p_tcb->param ) {
			powercapsule( p_tcb ) ;
		} else {
			enemy_destroy( p_tcb ) ;
		}
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}
