// -------------------------------------------------------------------------------- 
//!  @file  enemy02.c                                                               
//!  敵 02 タスク処理のソースファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "enemy02.h"


// -------------------------------------------------------------------------------- 
//!  敵 02 当たり処理                                                               
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy02_hit(HIT *p_self, HIT *p_rival)
{
	p_rival->proc(p_rival, p_self) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 02 タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy02( ENEMYTCB *p_tcb )
{
	p_tcb->obj.x -= toFp( g.bg.xofs ) ;
	if( p_tcb->obj.x < toFp( -16 ) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	p_tcb->obj.y -= toFp( g.bg.yofs ) ;

	if( p_tcb->timer0 ) {
		if( !--p_tcb->timer0 ) {
			obj_setAction(&p_tcb->obj, p_tcb->obj.action) ;
		}
		return ;
	}

	if( p_tcb->obj.status & OBJ_STATUS_ACTEND ) {
		p_tcb->timer0 = 336 ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 02 タスクを生成                                                             
//!                                                                                 
//!  @param [in] y      ｙ座標                                                      
//!  @param [in] param  パラメータ                                                  
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy02_create(int24f8_t y, uint16_t param)
{
	static const uint8_t	action[]={
								ACT_ENEMY02U,
								ACT_ENEMY02D,
								ACT_ENEMY02L,
								ACT_ENEMY02R
							} ;
	static const uint16_t	wait[]={
								16,
								240,
								240,
								254
							} ;
	ENEMYTCB				*p_tcb ;


	if( !g.enemy.remain ) {
		return ;
	}
	g.enemy.remain-- ;

	// タスク生成
	p_tcb               = task_create(NULL, enemy02, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.prio     = OBJ_PRIO1 | 0 ;
	p_tcb->obj.action   = action[param>>8] ;
	p_tcb->obj.x        = toFp( SCREEN_WIDTH ) ;
	p_tcb->obj.y        = y ;
	p_tcb->obj.hit.proc = enemy02_hit ;
	p_tcb->timer0       = wait[param>>8] ;
	p_tcb->param        = param ;
}
