// -------------------------------------------------------------------------------- 
//!  @file  task.h                                                                  
//!  タスク管理のヘッダファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __TASK_H__INCLUDED__
#define __TASK_H__INCLUDED__


typedef struct _hit					HIT ;							//!< 当たり情報
typedef struct _obj					OBJ ;							//!< オブジェクト情報


#include "hit.h"
#include "obj.h"

//! TCB ヘッダ
#define TCBHEAD( parent )																	\
		TCB							*p_prev ;						/*!< 前タスク TCB   */	\
		TCB							*p_next ;						/*!< 次タスク TCB   */	\
		parent						*p_parent ;						/*!< 親タスク TCB   */	\
		TASKPROC					taskProc;						/*!< タスク処理     */	\
		TASKPROC					pauseProc ;						/*!< タスク停止処理 */	\
		uint8_t						status ;						/*!< 状態           */	\
		uint8_t						prio ;							/*!< タスク優先度   */	\
		uint16_t					count0 ;						/*!< 汎用カウンタ０ */	\
		uint32_t					timer0 ;						/*!< 汎用タイマー０ */


typedef struct _tcb					TCB ;							//!< タスクコントロールブロック
typedef void (*TASKPROC)() ;										//!< タスク処理（typedef void (*TASKPROC)( TCB *p_tcb )）

	//!< システムモード
	enum {
		SYSTEM_MODE_TITLE,											//!< タイトル
		SYSTEM_MODE_POWERMETERSELECT,								//!< パワーメーター選択
		SYSTEM_MODE_OPTION,											//!< オプション
		SYSTEM_MODE_GAME,											//!< ゲーム
		SYSTEM_MODE_MISS,											//!< ゲーム中ミス
		SYSTEM_MODE_GAMEOVER,										//!< ゲームオーバー
		SYSTEM_MODE_MAX												//!< 最大数
	} ;

	//! TCB 状態
	enum {
		TCB_STATUS_NONEXISTENT		=	1 << 6,						//!< 未登録状態
		TCB_STATUS_DORMANT			=	1 << 7,						//!< 休止状態
	} ;

	//! タスクコントロールブロック
	struct _tcb {
		TCBHEAD( TCB )												//!< TCB ヘッダ
		uint32_t					work[TASK_WORK/4] ;				//!< 作業バッファ
	} ;

	//! システムタスク TCB
	typedef struct {
		TCBHEAD( TCB )												//!< TCB ヘッダ
		uint8_t						childCount ;					//!< 作成子タスク数
		uint8_t						cursor ;						//!< カーソル
		uint32_t					score ;							//!< 以前の得点
		uint8_t						powerCapsule ;					//!< 以前の取得パワーカプセル数
		uint8_t						powerMeterStatus ;				//!< 以前のパワーメーター状態
	} SYSTEMTCB ;

	//! 後処理タスク TCB
	typedef struct {
		TCBHEAD( TCB )												//!< TCB ヘッダ
	} POSTTCB ;


extern void           task_init( void ) ;
extern void * RAMFUNC task_create(TCB *p_parent, TASKPROC taskProc, uint8_t prio) ;
extern void   RAMFUNC task_destroy( TCB *p_tcb ) ;
extern void   RAMFUNC task_destroyPrio(uint8_t first, uint8_t last) ;
extern void * RAMFUNC task_getTcbPrio( uint8_t prio ) ;
extern void   RAMFUNC task_sleep( TCB *p_tcb ) ;
extern void   RAMFUNC task_exec( bool pause ) ;
extern int    RAMFUNC task_debug( void ) ;
extern void           task_setSystemMode( int mode ) ;


#endif//__TASK_H__INCLUDED__
