// -------------------------------------------------------------------------------- 
//!  @file  multiple_shadow.c                                                       
//!  マルチプル・シャドウタスク処理のソースファイル                                 
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "multiple_shadow.h"


// -------------------------------------------------------------------------------- 
//!  マルチプル・シャドウタスク                                                     
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセンマルチプルタスク TCB                              
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC multiple_shadow( VIXENMULTIPLETCB *p_tcb )
{
	VIXENTCB	*p_parent ;
	int16_t		*p_pos ;


	p_parent = p_tcb->p_parent ;

	// 移動
	p_pos        = &g.vixen.pos[(p_parent->posIndex+p_tcb->posIndex)&VIXEN_POS_MASK][0] ;
	p_tcb->obj.x = toFp( p_pos[0] ) ;
	p_tcb->obj.y = toFp( p_pos[1] ) ;

	// ショット
	p_parent->shotProc( p_tcb ) ;

	// ミサイル
	p_parent->missileProc( p_tcb ) ;

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  マルチプル・シャドウタスクを生成                                               
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC multiple_shadow_create( VIXENTCB *p_parent )
{
	VIXENMULTIPLETCB	*p_tcb ;
	uint8_t				posIndex ;
	int16_t				*p_pos ;


	posIndex = ++g.vixen.multipleCount * 8 ;
	p_pos    = &g.vixen.pos[(p_parent->posIndex+posIndex)&VIXEN_POS_MASK][0] ;

	p_tcb             = task_create((TCB *)p_parent, multiple_shadow, TASK_PRIO_MULTIPLE) ;
	p_tcb->obj.prio   = OBJ_PRIO2 | 4 ;
	p_tcb->obj.status = 0 ;
	p_tcb->obj.x      = toFp( p_pos[0] ) ;
	p_tcb->obj.y      = toFp( p_pos[1] ) ;
	p_tcb->posIndex   = posIndex ;

	obj_setAction(&p_tcb->obj, ACT_MULTIPLE) ;
}
