// -------------------------------------------------------------------------------- 
//!  @file  napalmmissile.c                                                         
//!  ナパーム・ミサイルタスク処理のソースファイル                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "napalmmissile.h"


static void RAMFUNC napalmmissile_040( VIXENWEAPONTCB *p_tcb ) ;
static void RAMFUNC napalmmissile_080( VIXENWEAPONTCB *p_tcb ) ;
static void RAMFUNC napalmmissile_0c0( VIXENWEAPONTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  ナパーム・ミサイル当たり処理                                                   
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC napalmmissile_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_tcb->taskProc = napalmmissile_080 ;
}


// -------------------------------------------------------------------------------- 
//!  ナパーム・ミサイルタスク処理                                                   
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC napalmmissile( VIXENWEAPONTCB *p_tcb )
{
	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		p_tcb->taskProc = napalmmissile_080 ;
		obj_setPattern(&p_tcb->obj, PAT_NAPALMMISSILE0+((p_tcb->angle+10)/16)) ;
		return ;
	}

	p_tcb->taskProc = napalmmissile_040 ;

	napalmmissile_040( p_tcb ) ;
}


static void RAMFUNC napalmmissile_040( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	y ;


	do {
		// ｙ方向移動
		y             = p_tcb->obj.y ;
		p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
		if( p_tcb->obj.y >= toFp( SCREEN_HEIGHT+8 ) ) {
			vixen_weapon_destroy( p_tcb ) ;
			return ;
		}

		// 着地
		if( bg_checkHitY(p_tcb->obj.x, y, p_tcb->obj.y, &p_tcb->obj.y) & BG_ATTR_WALL ) {
			p_tcb->taskProc = napalmmissile_080 ;
			break ;
		}

		// ｘ方向移動
		p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
		if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+16 ) ) {
			vixen_weapon_destroy( p_tcb ) ;
			return ;
		}

		// 壁
		if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
			p_tcb->taskProc = napalmmissile_080 ;
			break ;
		}

		// 角度
		p_tcb->speed += (FP_ONE / 16) ;

		if( p_tcb->angle < 54 ) {
			p_tcb->angle++ ;
			if( p_tcb->angle < 40 ) {
				p_tcb->angle++ ;
			}
		}
	} while( 0 ) ;

	obj_setPattern(&p_tcb->obj, PAT_NAPALMMISSILE0+((p_tcb->angle+10)/16)) ;
}


static void RAMFUNC napalmmissile_080( VIXENWEAPONTCB *p_tcb )
{
	p_tcb->taskProc     = napalmmissile_0c0 ;
	p_tcb->timer0       = 40 ;
	p_tcb->obj.hit.proc = hit_nothing ;

	obj_setAction(&p_tcb->obj, ACT_NAPARMEXPLOSION) ;
}


static void RAMFUNC napalmmissile_0c0( VIXENWEAPONTCB *p_tcb )
{
	if( !--p_tcb->timer0 ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	p_tcb->obj.x += (getCos( p_tcb->angle ) * 7 - toFp( g.bg.xofs )) ;
	p_tcb->obj.y += (getSin( p_tcb->angle ) * 7 - toFp( g.bg.yofs )) ;
	p_tcb->angle += 75 ;

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  ナパーム・ミサイルタスクを生成                                                 
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC napalmmissile_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;


	if( !(g.pad.conv&PAD_BTN( 2 )) ) {
		return ;
	}

	if( p_parent->weapon & VIXEN_WEAPON_MISSILE ) {
		return ;
	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, napalmmissile, TASK_PRIO_WEAPON) ;
	p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = p_parent->obj.x + toFp( 4 ) ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = napalmmissile_hit ;
	p_tcb->obj.hit.strength = 4 ;
	p_tcb->weapon           = VIXEN_WEAPON_MISSILE ;
	p_tcb->speed            = toFp( 2.5 ) ;
	p_tcb->angle            = 8 ;

	obj_setPattern(&p_tcb->obj, PAT_NAPALMMISSILE0+((p_tcb->angle+10)/16)) ;

	p_parent->weapon |= VIXEN_WEAPON_MISSILE ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_04) ;
	}
}
