// -------------------------------------------------------------------------------- 
//!  @file  ripplelaser.c                                                           
//!  リップル・レーザータスク処理のソースファイル                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "ripplelaser.h"


// -------------------------------------------------------------------------------- 
//!  リップル・レーザー当たり処理                                                   
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC ripplelaser_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_tcb->taskProc = vixen_weapon_destroy ;
}


// -------------------------------------------------------------------------------- 
//!  リップル・レーザータスク                                                       
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC ripplelaser( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	x ;

	// ｘ方向
	x             = p_tcb->obj.x ;
	p_tcb->obj.x += p_tcb->speed ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+32 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHitX(x, p_tcb->obj.y, p_tcb->obj.x, &p_tcb->obj.x) & BG_ATTR_WALL ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  リップル・レーザータスクを生成                                                 
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC ripplelaser_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;
	uint8_t			weapon ;


	if( !(g.pad.conv&PAD_BTN( 1 )) ) {
		return ;
	}

	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT0) ) {
		weapon = VIXEN_WEAPON_SHOOT0 ;
	} else
	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT1) ) {
		weapon = VIXEN_WEAPON_SHOOT1 ;
	} else {
		return ;
	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, ripplelaser, TASK_PRIO_WEAPON) ;
	p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = p_parent->obj.x + toFp( 4 ) ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = ripplelaser_hit ;
	p_tcb->obj.hit.strength = 20 ;
	p_tcb->weapon           = weapon ;
	p_tcb->speed            = toFp( 8 ) ;

	obj_setAction(&p_tcb->obj, ACT_RIPPLELASER) ;

	p_parent->weapon |= weapon ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_07) ;
	}
}
