// -------------------------------------------------------------------------------- 
//!  @file  shield.c                                                                
//!  シールドタスク処理のソースファイル                                             
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "shield.h"


// -------------------------------------------------------------------------------- 
//!  シールドダメージ                                                               
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセンタスク TCB                                        
//!  @retval     true   破壊                                                        
//!  @retval     false  非破壊                                                      
// -------------------------------------------------------------------------------- 
static inline bool shield_damage( VIXENTCB *p_tcb )
{
	if( --p_tcb->shieldRemain <= 0 ) {
		return true ;
	}

	// 0x2222 == 1/5/9/13
	if( (1<<p_tcb->shieldRemain) & 0x2222 ) {
		p_tcb->shieldAction += (ACT_SHIELDU1 - ACT_SHIELDU0) ;
	}

	return false ;
}


// -------------------------------------------------------------------------------- 
//!  シールドタスク当たり処理                                                       
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC shield_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	if( shield_damage( p_tcb->p_parent ) != false ) {
		p_tcb->obj.hit.flag = 0 ;
	}

	sound_requestSE(2, SE_30) ;
}


// -------------------------------------------------------------------------------- 
//!  シールドタスク                                                                 
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC shield( VIXENWEAPONTCB *p_tcb )
{
	VIXENTCB	*p_parent ;
	uint8_t		action, angle, attr ;
	int			diff ;


	p_parent = p_tcb->p_parent ;

	if( p_parent->shieldRemain <= 0 ) {
		vixen_weapon_destroy( p_tcb ) ;

		// WEAPON_SHIELD0 / WEAPON_SHIELD1 は必ず同時に破棄されるので無条件にパワーメーター有効
		g.powerMeter.status |= POWERMETER_STATUS_SHIELD ;
		return ;
	}

	if( !p_tcb->flag ) {	// ヴィクセン取り付け前

		angle = getAngle(getInt( p_tcb->obj.x ), getInt( p_tcb->obj.y ), getInt( p_parent->obj.x+p_tcb->xofs ), getInt( p_parent->obj.y+p_tcb->yofs )) ;
		diff  = (int8_t )(angle - p_tcb->angle) ;
		if( diff < -8 ) {
			diff = -8 ;
		} else
		if( diff >  8 ) {
			diff =  8 ;
		}

		p_tcb->angle    += diff ;
		p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
		p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;

		diff = getInt( p_tcb->obj.x - (p_parent->obj.x + p_tcb->xofs) ) ;
		if( (uint32_t )(diff+7) < (7*2) ) {
			diff = getInt( p_tcb->obj.y - (p_parent->obj.y + p_tcb->yofs) ) ;
			if( (uint32_t )(diff+7) < (7*2) ) {
				p_tcb->flag = 1 ;
			}
		}

	} else {				// ヴィクセン取り付け後

		p_tcb->obj.x = p_parent->obj.x + p_tcb->xofs ;
		p_tcb->obj.y = p_parent->obj.y + p_tcb->yofs ;

	}

	// 壁
	if( p_tcb->count0 ) {
		p_tcb->count0-- ;
	} else {
		attr  = bg_checkHit(p_tcb->obj.x-toFp( 6 ), p_tcb->obj.y-toFp( 6 )) ;
		attr |= bg_checkHit(p_tcb->obj.x+toFp( 6 ), p_tcb->obj.y-toFp( 6 )) ;
		attr |= bg_checkHit(p_tcb->obj.x-toFp( 6 ), p_tcb->obj.y+toFp( 6 )) ;
		attr |= bg_checkHit(p_tcb->obj.x+toFp( 6 ), p_tcb->obj.y+toFp( 6 )) ;

		if( attr & BG_ATTR_WALL ) {
			sound_requestSE(2, SE_30) ;

			if( shield_damage( p_tcb->p_parent ) != false ) {
				return ;
			}

			p_tcb->count0 = 15 ;
		}
	}

	// アクション更新
	action = p_tcb->action + p_parent->shieldAction ;
	if( action != p_tcb->obj.action ) {
		obj_setAction(&p_tcb->obj, action) ;
	} else {
		obj_updateAction( &p_tcb->obj ) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  シールドタスクを生成                                                           
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC shield_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;

	// シールド上のタスク生成
	p_tcb                   = task_create((TCB *)p_parent, shield, TASK_PRIO_SHIELD) ;
	p_tcb->obj.prio         = OBJ_PRIO2 | 0 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = toFp( SCREEN_WIDTH + 8 ) ;
	p_tcb->obj.y            = toFp( 0 ) ;
	p_tcb->obj.hit.proc     = shield_hit ;
	p_tcb->obj.hit.strength = 8 ;
	p_tcb->speed            = toFp(  4 ) ;
	p_tcb->xofs             = toFp( 24 ) ;
	p_tcb->yofs             = toFp( -8 ) ;
	p_tcb->angle            = getAngle(getInt( p_tcb->obj.x ), getInt( p_tcb->obj.y ), getInt( p_parent->obj.x+p_tcb->xofs ), getInt( p_parent->obj.y+p_tcb->yofs )) ;
	p_tcb->action           = ACT_SHIELDU0 ;
	p_tcb->flag             = 0 ;

	obj_setAction(&p_tcb->obj, ACT_SHIELDU0) ;

	// シールド下のタスク生成
	p_tcb                   = task_create((TCB *)p_parent, shield, TASK_PRIO_SHIELD) ;
	p_tcb->obj.prio         = OBJ_PRIO2 | 0 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = toFp( SCREEN_WIDTH  + 8 ) ;
	p_tcb->obj.y            = toFp( SCREEN_HEIGHT - 1 ) ;
	p_tcb->obj.hit.proc     = shield_hit ;
	p_tcb->obj.hit.strength = 8 ;
	p_tcb->speed            = toFp(  4 ) ;
	p_tcb->xofs             = toFp( 24 ) ;
	p_tcb->yofs             = toFp(  8 ) ;
	p_tcb->angle            = getAngle(getInt( p_tcb->obj.x ), getInt( p_tcb->obj.y ), getInt( p_parent->obj.x+p_tcb->xofs ), getInt( p_parent->obj.y+p_tcb->yofs )) ;
	p_tcb->action           = ACT_SHIELDD0 ;
	p_tcb->flag             = 0 ;

	obj_setAction(&p_tcb->obj, ACT_SHIELDD0) ;

	p_parent->weapon      |= (VIXEN_WEAPON_SHIELD0 | VIXEN_WEAPON_SHIELD1) ;
	p_parent->shieldRemain = 15 ;
	p_parent->shieldAction = 0 ;
}
