// -------------------------------------------------------------------------------- 
//!  @file  tailbeam.c                                                              
//!  テイル・ビームタスク処理のソースファイル                                       
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "tailbeam.h"


// -------------------------------------------------------------------------------- 
//!  テイル・ビーム当たり処理                                                       
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC tailbeam_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_tcb->taskProc = vixen_weapon_destroy ;
}


// -------------------------------------------------------------------------------- 
//!  テイル・ビームタスク処理                                                       
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC tailbeam( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	x ;

	// ｘ方向移動
	x             = p_tcb->obj.x ;
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( (uint32_t )(getInt( p_tcb->obj.x )+8) >= (SCREEN_WIDTH+(8*2)) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHitX(x, p_tcb->obj.y, p_tcb->obj.x, &p_tcb->obj.x) & BG_ATTR_WALL ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  テイル・ビームタスクを生成                                                     
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC tailbeam_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;


	if( !(g.pad.rapid&PAD_BTN( 1 )) ) {
		return ;
	}

	if( (p_parent->weapon&(VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1)) == (VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1) ) {
		return ;
	}

	// 前ビームのタスク生成
	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT0) ) {
		p_tcb                   = task_create((TCB *)p_parent, tailbeam, TASK_PRIO_WEAPON) ;
		p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
		p_tcb->obj.status       = 0 ;
		p_tcb->obj.x            = p_parent->obj.x + toFp( 4 ) ;
		p_tcb->obj.y            = p_parent->obj.y ;
		p_tcb->obj.hit.proc     = tailbeam_hit ;
		p_tcb->obj.hit.strength = 8 ;
		p_tcb->weapon           = VIXEN_WEAPON_SHOOT0 ;
		p_tcb->speed            = toFp( 6 ) ;
		p_tcb->angle            = 0 ;

		obj_setAction(&p_tcb->obj, ACT_TAILBEAM0) ;
	}

	// 後ビームのタスク生成
	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT1) ) {
		p_tcb                   = task_create((TCB *)p_parent, tailbeam, TASK_PRIO_WEAPON) ;
		p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
		p_tcb->obj.status       = 0 ;
		p_tcb->obj.x            = p_parent->obj.x - toFp( 4 ) ;
		p_tcb->obj.y            = p_parent->obj.y ;
		p_tcb->obj.hit.proc     = tailbeam_hit ;
		p_tcb->obj.hit.strength = 8 ;
		p_tcb->weapon           = VIXEN_WEAPON_SHOOT1 ;
		p_tcb->speed            = toFp( 6 ) ;
		p_tcb->angle            = 128 ;

		obj_setAction(&p_tcb->obj, ACT_TAILBEAM1) ;
	}

	p_parent->weapon |= (VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1) ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_05) ;
	}
}
