// -------------------------------------------------------------------------------- 
//!  @file  updouble.c                                                              
//!  アップ・ダブルタスク処理のソースファイル                                       
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "updouble.h"


// -------------------------------------------------------------------------------- 
//!  アップ・ダブル当たり処理                                                       
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC updouble_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_tcb->taskProc = vixen_weapon_destroy ;
}


// -------------------------------------------------------------------------------- 
//!  アップ・ダブルタスク                                                           
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC updouble( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	x ;

	// ｘ方向移動
	x             = p_tcb->obj.x ;
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.x >= ((SCREEN_WIDTH+16)*FP_ONE) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// ｙ方向移動
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.y <= (-16*FP_ONE) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHitX(x, p_tcb->obj.y, p_tcb->obj.x, &p_tcb->obj.x) & BG_ATTR_WALL ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  アップ・ダブルタスクを生成                                                     
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC updouble_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;


	if( !(g.pad.rapid&PAD_BTN( 1 )) ) {
		return ;
	}

	if( (p_parent->weapon&(VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1)) == (VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1) ) {
		return ;
	}

	// 前ビームのタスク生成
	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT0) ) {
		p_tcb                   = task_create((TCB *)p_parent, updouble, TASK_PRIO_WEAPON) ;
		p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
		p_tcb->obj.status       = 0 ;
		p_tcb->obj.x            = p_parent->obj.x + toFp( 4 ) ;
		p_tcb->obj.y            = p_parent->obj.y ;
		p_tcb->obj.hit.proc     = updouble_hit ;
		p_tcb->obj.hit.strength = 8 ;
		p_tcb->weapon           = VIXEN_WEAPON_SHOOT0 ;
		p_tcb->speed            = toFp( 6 ) ;
		p_tcb->angle            = 0 ;

		obj_setAction(&p_tcb->obj, ACT_BEAM) ;
	}

	// 後ビームのタスク生成
	if( !(p_parent->weapon&VIXEN_WEAPON_SHOOT1) ) {
		p_tcb                   = task_create((TCB *)p_parent, updouble, TASK_PRIO_WEAPON) ;
		p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
		p_tcb->obj.status       = 0 ;
		p_tcb->obj.x            = p_parent->obj.x + toFp( 4 ) ;
		p_tcb->obj.y            = p_parent->obj.y ;
		p_tcb->obj.hit.proc     = updouble_hit ;
		p_tcb->obj.hit.strength = 8 ;
		p_tcb->weapon           = VIXEN_WEAPON_SHOOT1 ;
		p_tcb->speed            = toFp( 6 ) ;
		p_tcb->angle            = -32 ;

		obj_setAction(&p_tcb->obj, ACT_UPDOUBLE) ;
	}

	p_parent->weapon |= (VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1) ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_03) ;
	}
}
