// -------------------------------------------------------------------------------- 
//!  @file  vixen.c                                                                 
//!  ヴィクセンタスク処理のソースファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "beam.h"
#include "missile.h"
#include "photonmissile.h"
#include "napalmmissile.h"
#include "twowaymissile.h"
#include "updouble.h"
#include "tailbeam.h"
#include "laser.h"
#include "ripplelaser.h"
#include "multiple_shadow.h"
#include "multiple_fixed.h"
#include "multiple_rolling.h"
#include "shield.h"
#include "forcefield.h"


	enum {
		START_VIXEN_X				=	100,						//!< 開始時ヴィクセンｘ座標
		START_VIXEN_Y				=	112,						//!< 開始時ヴィクセンｙ座標
		MULTIPLE_XOFS				=	-8							//!< マルチプル表示ｘ座標オフセット
	} ;

	//! パレット更新タスク TCB
	typedef struct {
		TCBHEAD( VIXENTCB )											//!< TCB ヘッダ
		uint16_t					*p_palette ;					//!< パレットアドレス
		uint16_t					color ;							//!< 色
	} PALETTETCB ;


static TASKPROC						m_weaponProc[5] ;				//!< 武器処理
static TASKPROC						m_multipleProc ;				//!< マルチプルタスク処理
static TASKPROC						m_shieldProc ;					//!< シールドタスク処理
static uint8_t						m_powerUpSE[6] ;				//!< パワーアップ SE


static void RAMFUNC vixen_040( VIXENTCB *p_tcb ) ;
static void RAMFUNC vixen_weapon_effect_040( VIXENWEAPONTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  パレット更新 VBlank 処理                                                       
//!                                                                                 
//!  @param [in,out] p_tcb  パレットタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC updatePalette_vblank( PALETTETCB *p_tcb )
{
	g.vram.pal[PALPOS_VIXEN][1] = p_tcb->p_palette[1] ;
	g.vram.pal[PALPOS_VIXEN][2] = p_tcb->p_palette[2] ;
	g.vram.pal[PALPOS_VIXEN][3] = p_tcb->p_palette[3] ;

	if( g.powerMeter.playerType == 2 ) {
		g.vram.pal[PALPOS_VIXEN][6] = p_tcb->color ;
	}
}


// -------------------------------------------------------------------------------- 
//!  パレット更新タスク                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  パレットタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void updatePalette( PALETTETCB *p_tcb )
{
	int		index, color ;


	index = (g.count2 & 1) * 4 ;
	if( p_tcb->p_parent->speedCount ) {
		index |= 8 ;
	}

	p_tcb->p_palette = &g.vram.pal[PALPOS_VIXENB][index] ;

	color = getInt( getSin( g.count1 & 127 ) * 18 ) ;
	if( color > 15 ) {
		color = 15 ;
	}
	p_tcb->color = color ;

	addVblankProc(updatePalette_vblank, p_tcb) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセンを移動                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  ヴィクセンタスク TCB                                    
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static inline void move( VIXENTCB *p_tcb )
{
	static const uint8_t	m_vixenAngle[]={	//!< ヴィクセン角度
								1,				//
								128,			// 左
								0,				// 右
								1,				//
								192,			// 上
								160,			// 左上
								224,			// 右上
								1,				//
								64,				// 下
								96,				// 左下
								32,				// 右下
								1,				//
								1,				// 上
								1,				// 左上
								1,				// 右上
								1				//
							} ;
	uint8_t					angle ;
	int16_t					*p_pos ;
	uint16_t				pad ;

	// 移動
	angle = m_vixenAngle[(g.pad.norm>>PAD_DIR_)&15] ;
	pad   = 0 ;
	if( angle != 1 ) {
		pad = g.pad.norm & (PAD_LEFT|PAD_RIGHT|PAD_UP|PAD_DOWN) ;

		p_tcb->obj.x += getInt( getCos( angle ) * p_tcb->speed ) ;
		if( p_tcb->obj.x < toFp( 32 ) ) {
			p_tcb->obj.x = toFp( 32 ) ;
		} else
		if( p_tcb->obj.x > toFp( SCREEN_WIDTH-32 ) ) {
			p_tcb->obj.x = toFp( SCREEN_WIDTH-32 ) ;
		}

		p_tcb->obj.y += getInt( getSin( angle ) * p_tcb->speed ) ;
		if( p_tcb->obj.y < toFp( 16 ) ) {
			p_tcb->obj.y = toFp( 16 ) ;
		} else
		if( p_tcb->obj.y > toFp( SCREEN_HEIGHT-16 ) ) {
			p_tcb->obj.y = toFp( SCREEN_HEIGHT-16 ) ;
		}
	}

	// 傾き
	if( pad & (PAD_UP|PAD_DOWN) ) {
		if( pad & PAD_UP ) {
			if( p_tcb->tilt > -39 ) {
				p_tcb->tilt-- ;
			}
		} else {
			if( p_tcb->tilt <  39 ) {
				p_tcb->tilt++ ;
			}
		}
	} else {
		if( p_tcb->tilt < 0 ) {
			p_tcb->tilt++ ;
		} else
		if( p_tcb->tilt > 0 ) {
			p_tcb->tilt-- ;
		}
	}

	// マルチプル・シャドウ用座標
	if( pad ) {
		p_tcb->posIndex-- ;
		p_pos    = &g.vixen.pos[p_tcb->posIndex&VIXEN_POS_MASK][0] ;
		p_pos[0] = getInt( p_tcb->obj.x ) + MULTIPLE_XOFS ;
		p_pos[1] = getInt( p_tcb->obj.y ) ;
	}

	g.player.x = p_tcb->obj.x ;
	g.player.y = p_tcb->obj.y ;
}


// -------------------------------------------------------------------------------- 
//!  パワーアップ                                                                   
//!                                                                                 
//!  @param [in,out] p_tcb  ヴィクセンタスク TCB                                    
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static inline void powerUp( VIXENTCB *p_tcb )
{
	if( !(g.pad.conv&PAD_BTN( 0 )) ) {
		return ;
	}

	// パワーアップ済みなら無視
	if( !(g.powerMeter.status&(1<<g.powerCapsule)) ) {
		return ;
	}

	switch( g.powerCapsule ) {
		case 1:		// SPEEDUP
		{
			if( ++p_tcb->speedCount == 5 ) {
				g.powerMeter.status &= ~POWERMETER_STATUS_SPEED ;
			}

			p_tcb->speed = (p_tcb->speedCount + 1) * toFp( 1.1 ) ;
			break ;
		}

		case 2:		// MISSILE
		{
			g.powerMeter.status &= ~POWERMETER_STATUS_MISSILE ;

			p_tcb->missileProc = m_weaponProc[0] ;
			break ;
		}

		case 3:		// DOUBLE
		{
			g.powerMeter.status = (g.powerMeter.status & ~POWERMETER_STATUS_DOUBLE) | POWERMETER_STATUS_LASER ;

			p_tcb->shotProc = m_weaponProc[1] ;
			break ;
		}

		case 4:		// LASER
		{
			g.powerMeter.status = (g.powerMeter.status & ~POWERMETER_STATUS_LASER) | POWERMETER_STATUS_DOUBLE ;

			p_tcb->shotProc = m_weaponProc[2] ;
			break ;
		}

		case 5:		// OPTION
		{
			m_multipleProc( p_tcb ) ;

			if( g.vixen.multipleCount == 2 ) {
				g.powerMeter.status &= ~POWERMETER_STATUS_MULTIPLE ;
			}
			break ;
		}

//		case 6:		// ？
		default:
		{
			g.powerMeter.status &= ~POWERMETER_STATUS_SHIELD ;

			m_shieldProc( p_tcb ) ;
			break ;
		}
	}

	sound_requestSE(0, m_powerUpSE[g.powerCapsule-1]) ;
	sound_requestSE(3, SE_02) ;

	g.powerCapsule = 0 ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセン破壊                                                                 
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセンタスク TCB                                        
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC vixen_destroy( VIXENTCB *p_tcb )
{
	p_tcb->taskProc = vixen_040 ;
	p_tcb->timer0   = 30 ;

	if( g.powerMeter.playerType != 3 ) {
		obj_setAction(&p_tcb->obj, ACT_EXPLOSIONVIXEN0) ;
	} else {
		obj_setAction(&p_tcb->obj, ACT_EXPLOSIONVIXEN1) ;
	}

	sound_requestSE(0, SE_20) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセン当たり処理                                                           
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC vixen_hit(HIT *p_self, HIT *p_rival)
{
	VIXENTCB	*p_tcb=MEMBERSTRUCT(VIXENTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	vixen_destroy( p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセンタスク                                                               
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセンタスク TCB                                        
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC vixen( VIXENTCB *p_tcb )
{
	// 変換
	g.pad.conv = (((g.pad.norm>>g.cfg.button[0])&1)<<(PAD_BTN_+0)) |
	             (((g.pad.norm>>g.cfg.button[1])&1)<<(PAD_BTN_+1)) |
	             (((g.pad.norm>>g.cfg.button[2])&1)<<(PAD_BTN_+2)) ;

	// 連射
	if( g.cfg.rapid ) {
		g.pad.rapid = 0 ;
		if( g.pad.norm & PAD_BTN( g.cfg.button[1] ) ) {
			if( !p_tcb->count0 ) {
				g.pad.rapid = PAD_BTN( 1 ) ;
			}
			p_tcb->count0 = (p_tcb->count0 + 1) & 3 ;
		} else {
			p_tcb->count0 = 0 ;
		}
	} else {
		g.pad.rapid = (((g.pad.once>>g.cfg.button[1])&1)<<(PAD_BTN_+1)) ;
	}

	// 移動
	move( p_tcb ) ;

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_destroy( p_tcb ) ;
		return ;
	}

	// ショット
	p_tcb->shotProc( p_tcb ) ;

	// ミサイル
	p_tcb->missileProc( p_tcb ) ;

	// パワーアップ
	powerUp( p_tcb ) ;

	obj_setPattern(&p_tcb->obj, PAT_VIXEN0+((p_tcb->tilt+40)/16)) ;
}


static void RAMFUNC vixen_040( VIXENTCB *p_tcb )
{
	if( !--p_tcb->timer0 ) {
		task_setSystemMode( SYSTEM_MODE_MISS ) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセンタスクを生成                                                         
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC vixen_create( SYSTEMTCB *p_parent )
{
	static const TASKPROC	weaponProc[][3]={
								{ missile_create,       updouble_create, laser_create       },
								{ photonmissile_create, updouble_create, laser_create       },
								{ napalmmissile_create, tailbeam_create, ripplelaser_create },
								{ twowaymissile_create, tailbeam_create, ripplelaser_create }
							},
							multipleProc[]={
								multiple_shadow_create,
								multiple_fixed_create,
								multiple_rolling_create
							},
							shieldProc[]={
								shield_create,
								forcefield_create
							} ;
	static const uint8_t	weaponSE[][3]={
								{ SE_41, SE_45, SE_47 },
								{ SE_44, SE_45, SE_47 },
								{ SE_42, SE_46, SE_48 },
								{ SE_43, SE_46, SE_48 }
							},
							multipleSE[]={
								SE_49, SE_49, SE_49
							},
							shieldSE[]={
								SE_50, SE_51
							} ;
	VIXENTCB				*p_tcb ;
	int						index ;


	DBGASSERT( sizeof( PALETTETCB )     <= sizeof( TCB ) )
	DBGASSERT( sizeof( VIXENTCB )       <= sizeof( TCB ) )
	DBGASSERT( sizeof( VIXENWEAPONTCB ) <= sizeof( TCB ) )

	// 初期化
	g.vixen.multipleCount = 0 ;
	m_weaponProc[0]       = weaponProc[g.powerMeter.playerType][0] ;
	m_weaponProc[1]       = weaponProc[g.powerMeter.playerType][1] ;
	m_weaponProc[2]       = weaponProc[g.powerMeter.playerType][2] ;
	m_weaponProc[3]       = multipleProc[g.powerMeter.multiple] ;
	m_weaponProc[4]       = shieldProc[g.powerMeter.shield] ;
	m_powerUpSE[0]        = SE_40 ;
	m_powerUpSE[1]        = weaponSE[g.powerMeter.playerType][0] ;
	m_powerUpSE[2]        = weaponSE[g.powerMeter.playerType][1] ;
	m_powerUpSE[3]        = weaponSE[g.powerMeter.playerType][2] ;
	m_powerUpSE[4]        = multipleSE[g.powerMeter.multiple] ;
	m_powerUpSE[5]        = shieldSE[g.powerMeter.shield] ;
	m_multipleProc        = m_weaponProc[3] ;
	m_shieldProc          = m_weaponProc[4] ;

	for(index=0; index<VIXEN_POS_COUNT; index++) {
		g.vixen.pos[index][0] = START_VIXEN_X + MULTIPLE_XOFS ;
		g.vixen.pos[index][1] = START_VIXEN_Y ;
	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, vixen, TASK_PRIO_PLAYER) ;
	p_tcb->obj.prio         = OBJ_PRIO1 | 15 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = toFp( START_VIXEN_X ) ;
	p_tcb->obj.y            = toFp( START_VIXEN_Y ) ;
	p_tcb->obj.hit.proc     = vixen_hit ;
	p_tcb->obj.hit.strength = 1 ;
	p_tcb->shotProc         = beam_create ;
	p_tcb->missileProc      = task_sleep ;
	p_tcb->count0           = 0 ;
	p_tcb->weapon           = 0 ;
	p_tcb->tilt             = 0 ;
	p_tcb->speedCount       = 0 ;
	p_tcb->speed            = toFp( 1 ) ;
	p_tcb->posIndex         = 0 ;

	// パレット更新タスク生成
	task_create((TCB *)p_tcb, updatePalette, TASK_PRIO_PLAYER+1) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセン武器タスク破棄                                                       
//!                                                                                 
//!  @param [in,out] p_tcb  ヴィクセン武器タスク TCB                                
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void RAMFUNC vixen_weapon_destroy( VIXENWEAPONTCB *p_tcb )
{
	p_tcb->p_parent->weapon &= ~p_tcb->weapon ;

	task_destroy( (TCB *)p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  ヴィクセン武器タスクのエフェクト表示                                           
//!                                                                                 
//!  @param [in,out] p_tcb   ヴィクセン武器タスク TCB                               
//!  @param [in,out] action  アクション番号                                         
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC vixen_weapon_effect(VIXENWEAPONTCB *p_tcb, uint8_t action)
{
	p_tcb->taskProc    = vixen_weapon_effect_040 ;
	p_tcb->obj.status &= ~OBJ_STATUS_SPRPRIO ;

	obj_setAction(&p_tcb->obj, action) ;
}


static void RAMFUNC vixen_weapon_effect_040( VIXENWEAPONTCB *p_tcb )
{
	if( p_tcb->obj.status & OBJ_STATUS_ACTEND ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	p_tcb->obj.x -= toFp( g.bg.xofs ) ;
	p_tcb->obj.y -= toFp( g.bg.yofs ) ;

	obj_updateAction( &p_tcb->obj ) ;
}
